package net.stepniak.api.picheese.config;

import com.sun.jersey.spi.spring.container.servlet.SpringServlet;
import net.stepniak.api.config.ApplicationContext;
import net.stepniak.api.filter.ApiOriginFilter;
import net.stepniak.api.utils.StaticServlet;
import org.springframework.web.WebApplicationInitializer;
import org.springframework.web.context.ContextLoaderListener;
import org.springframework.web.context.request.RequestContextListener;
import org.springframework.web.context.support.AnnotationConfigWebApplicationContext;
import org.springframework.web.servlet.DispatcherServlet;
import org.springframework.web.util.Log4jConfigListener;

import javax.servlet.FilterRegistration;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRegistration;

public class PicheeseApplicationInitializer implements WebApplicationInitializer {

  @Override
  public void onStartup(ServletContext servletContext) throws ServletException {
    // Load application context
    AnnotationConfigWebApplicationContext rootContext = new AnnotationConfigWebApplicationContext();
    rootContext.register(ApplicationContext.class);
    rootContext.setDisplayName("api");
    rootContext.scan("net.stepniak");

    boolean isDev = false;

    //set default profile to dev
    if (rootContext.getEnvironment().getDefaultProfiles().length < 1) {
      rootContext.getEnvironment().setDefaultProfiles("dev");
    }

    String[] profiles = rootContext.getEnvironment().getActiveProfiles();

    if (profiles.length < 1) {
      rootContext.getEnvironment().setActiveProfiles("dev");
      isDev = true;
    } else {
      if (profiles[0] == "dev") {
        isDev = true;
      }
    }

    // Add context loader listener
    servletContext.addListener(new Log4jConfigListener());
    servletContext.addListener(new ContextLoaderListener(rootContext));
    servletContext.addListener(new RequestContextListener());

    // Declare dispatcher servlet
    ServletRegistration.Dynamic dispatcher = servletContext.addServlet("apiDispatcher", new SpringServlet());
    dispatcher.setInitParameter("com.sun.jersey.spi.container.ContainerResponseFilters", "com.sun.jersey.api.container.filter.GZIPContentEncodingFilter");
    dispatcher.setInitParameter("com.sun.jersey.config.property.packages", "net.stepniak");

    //dont use POJOMappingFeature!!
    dispatcher.setInitParameter("com.sun.jersey.api.json.POJOMappingFeature", "false");

    String loggingFilter = "";

    if (isDev) {
      dispatcher.setInitParameter("com.sun.jersey.spi.container.ResourceFilters", "com.sun.jersey.api.container.filter.ResourceDebuggingFilterFactory");
      loggingFilter = ";com.sun.jersey.api.container.filter.LoggingFilter";
    }

    dispatcher.setInitParameter("com.sun.jersey.spi.container.ContainerRequestFilters", "com.sun.jersey.api.container.filter.PostReplaceFilter" + loggingFilter);

    dispatcher.setInitParameter("api.version", "0.6");
    dispatcher.setInitParameter("com.sun.jersey.config.feature.Trace", "false");  //debug
    dispatcher.addMapping("/api/picheese/*", "/storage/*");
    dispatcher.setLoadOnStartup(1);

    ServletRegistration.Dynamic jsonDispatcher = servletContext.addServlet("jsonDispatcher", new StaticServlet());
    jsonDispatcher.setInitParameter("defaultMimeType", "application/json");
    jsonDispatcher.addMapping("/picheese/*");
    jsonDispatcher.setLoadOnStartup(1);

    ServletRegistration.Dynamic defaultDispatcher = servletContext.addServlet("defaultDispatcher", new DispatcherServlet(rootContext));
    defaultDispatcher.addMapping("/");
    defaultDispatcher.setLoadOnStartup(1);

    // Register Spring security filter
    FilterRegistration.Dynamic apiOriginFilter = servletContext.addFilter("ApiOriginFilter", ApiOriginFilter.class);
    apiOriginFilter.addMappingForUrlPatterns(null, false, "/api/picheese/*");
    apiOriginFilter.setInitParameter("com.sun.jersey.config.feature.FilterForwardOn404", "true");
  }

}
