package net.stepniak.api.picheese.controller;

import net.stepniak.api.error.exception.ServerResponseException;
import net.stepniak.api.picheese.logic.ImageLogic;
import net.stepniak.api.picheese.repository.ImageDAO;
import net.stepniak.api.picheese.services.PicheeseBaseService;
import net.stepniak.picheese.error.server.ServerErrorType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.core.Response;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


@Path("images")
@Service
public class ImageController extends PicheeseBaseService<ImageLogic> {
  static Logger logger = LoggerFactory.getLogger(ImageController.class);
  static Pattern pattern = Pattern.compile("(.+)\\.(jpe?g|png|gif)$", Pattern.CASE_INSENSITIVE);

  public ImageController() {
    logger.info("ImageController constructor()");
  }

  public ImageDAO getImageDAO() {
    return (ImageDAO) getDao();
  }

  @Autowired
  public void setImageDAO(ImageDAO dao) {
    setDao(dao);
  }

//TODO rewrite to ImageContent
//
//  @PUT
//  @Consumes("image/*")
//  public Response uploadImage(byte[] data) throws IOException, ImageException {
//
//    validateSessionId(""); //validate sessionId
//
//    ImageLogic imageLogic = new ImageLogic();
//    imageLogic.upload(data);
//    imageLogic.setMimeType(request.getContentType());
//
//    getImageDAO().create(imageLogic);
//
//    logger.info("ImageController uploaded(\"{}\")", imageLogic.getId());
//
//    return Response
//      .status(Response.Status.CREATED)
//      .entity(getJsonStringFromEntity(imageLogic.getPojos()))
//      .location(UriBuilder.fromPath("/" + imageLogic.getId()).build())
//      .build();
//
////    Response response = create(imageLogic, uriInfo);
//
////    ImageStorage imageStorage = new ImageStorage(request.getRealPath(""));
////    imageStorage.saveImage(imageLogic);
////    return response;
//  }
//
//
//  @PUT
//  @Consumes(MediaType.MULTIPART_FORM_DATA)
//  public Response createWithImgByFormMultipart(
//    @FormDataParam("imgFile") FormDataContentDisposition fileDetail,
//    @FormDataParam("imgFile") InputStream fileContent
//  ) throws IOException, ImageException {
//
//    if (fileContent == null) {
//      throw new BadRequestResponseException(uriInfo, ApiError.BAD_REQUEST, new Exception("Missing imgFile param value."));
//    }
//
//    ByteArrayOutputStream buffer = new ByteArrayOutputStream();
//    int nRead;
//    byte[] data = new byte[16384];
//    byte[] imageInByte = null;
//
//    try {
//      while ((nRead = fileContent.read(data, 0, data.length)) != -1) {
//        buffer.write(data, 0, nRead);
//      }
//      buffer.flush();
//      imageInByte = buffer.toByteArray();
//      buffer.close();
//    } catch (IOException e) {
//      throw new InternalServerResponseException(uriInfo, ApiError.PARAM_IMAGE_BASE64, e);
//    }
//
//    return uploadImage(imageInByte);
//  }

  @GET
  @Path("{id}")
  public Response getImage(@PathParam("id") Long id) {
    //dont check sessionId

    logger.info("ImageController getImage(\"{}\")", id);
    return retrieve(id, uriInfo);
  }

  @GET
  @Path("{id}/{filename}")
  public Response getBinaryData(@PathParam("id") Long id, @PathParam("filename") String filename) {
    //dont check sessionId
    ImageLogic imageLogic = getImageDAO().retrieve(id);

    if (imageLogic == null) {
      throw new ServerResponseException(uriInfo, ServerErrorType.NOT_FOUND);
    }

    Matcher m = pattern.matcher(filename);

    if (!m.matches()) {
      throw new ServerResponseException(uriInfo, ServerErrorType.BAD_REQUEST);
    }

    byte[] data = {};

    if (m.group(1).equals("normal")) {
      data = imageLogic.getNormal();
    } else if (m.group(1).equals("thumbnail")) {
      data = imageLogic.getThumbnail();
    } else {
      data = imageLogic.getOriginal();
    }

    logger.info("ImageController getBinaryData(\"{}\", \"{}\")", id, filename);

    return Response
      .ok(data)
      .type(imageLogic.getMimeType())
      .build();
  }
}
