package net.stepniak.api.picheese.controller;

import net.stepniak.api.error.exception.ServerResponseException;
import net.stepniak.api.picheese.logic.UserLogic;
import net.stepniak.api.picheese.repository.UserDAO;
import net.stepniak.api.picheese.services.PicheeseBaseService;
import net.stepniak.api.picheese.validator.UserValidator;
import net.stepniak.picheese.error.server.ServerErrorType;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@Path("users")
@Produces(MediaType.APPLICATION_JSON + "; charset=utf-8")
//@Produces(MediaType.APPLICATION_JSON)
@Service
public class UserController extends PicheeseBaseService<UserLogic> {
  static Logger logger = LoggerFactory.getLogger(UserController.class);

  public UserController() {
    logger.info("UserController constructor()");
  }

  public UserDAO getUserDAO() {
    return (UserDAO) getDao();
  }

  @Autowired
  public void setUserDAO(UserDAO dao) {
    setDao(dao);
  }

  @POST
  @Consumes(MediaType.APPLICATION_FORM_URLENCODED)
  public Response createExternalByForm(
    @FormParam("key") String key,
    @FormParam("email") String email,
    @FormParam("userName") String userName
  ) {
    logger.info("UserController createExternalByForm(\"{}\", \"{}\", \"{}\", \"{}\")", key, email, userName);
    return createUser(key, email, userName);
  }

  @POST
  @Consumes(MediaType.APPLICATION_JSON)
  public Response createExternalByJson(
    JSONObject jsonObject
  ) {

    String key = null;
    String email = null;
    String userName = null;

    try {
      key = jsonObject.getString("key");
    } catch (JSONException e) {
      //do nothing
    }

    try {
      email = jsonObject.getString("email");
      userName = jsonObject.getString("userName");
    } catch (JSONException e) {
      throw new ServerResponseException(uriInfo, ServerErrorType.BAD_REQUEST, e);
    }

    logger.info("UserController createExternalByJson(\"{}\", \"{}\", \"{}\", \"{}\")", key, email, userName);
    return createUser(key, email, userName);
  }

  @POST
  public Response createByQueryString(
    @QueryParam("key") String key,
    @QueryParam("email") String email,
    @QueryParam("userName") String userName
  ) {
    logger.info("UserController createByQueryString(\"{}\", \"{}\", \"{}\", \"{}\")", key, email, userName);
    return createUser(key, email, userName);
  }

  private Response createUser(String key, String email, String userName) {
    validateApiKey(key);

    UserValidator.validateEmail(email, uriInfo);
    UserValidator.validateUserName(userName, uriInfo);

    UserLogic userLogic = UserLogic.createUser(userName, email);
    return save(userLogic, uriInfo);
  }

  @GET
  @Path("/{id}")
  public Response get(@PathParam("id") String id, @QueryParam("sessionId") String sessionId) {
    validateSessionId(sessionId);
    //TODO get only for owner (session verify)
    return retrieve(id, uriInfo);
  }
}
