package net.stepniak.api.picheese.logic;

import net.stepniak.api.entities.SimpleEntity;
import net.stepniak.api.image.ImageContent;
import net.stepniak.api.image.MimeType;
import net.stepniak.api.storage.ImageStorage;
import net.stepniak.picheese.PicheeseConst;
import net.stepniak.picheese.error.server.ServerErrorType;
import net.stepniak.picheese.error.server.exception.PhotoException;
import net.stepniak.picheese.pojos.v1.Photo;
import net.stepniak.picheese.pojos.v1.PhotoRating;
import net.stepniak.picheese.pojos.v1.PhotoUrl;
import net.stepniak.picheese.pojos.v1.User;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import java.io.IOException;


@Entity
@Table(name = "photo")
public class PhotoLogic extends SimpleEntity {
  @ManyToOne
  private UserLogic userLogic;
  private int score;

  @Column(nullable = false, length = PicheeseConst.PHOTOS_TITLE_LENGHT)
  private String title;

  @Column(nullable = false, length = PicheeseConst.PHOTOS_DESCRIPTION_LENGHT)
  private String description;

  @Column(nullable = false, length = 10)
  private String mimeType;

  private transient ImageContent imageContent;

  public PhotoLogic() {
  }

  public PhotoLogic(UserLogic userLogic) {
    this.userLogic = userLogic;
  }

  public PhotoRating getRating() {
    return new PhotoRating(this.score);
  }

  public String getDescription() {
    return description;
  }

  public String getTitle() {
    return title;
  }

  public UserLogic getUserLogic() {
    return userLogic;
  }

  public void setDescription(String description) throws PhotoException {
    if (description.length() > PicheeseConst.PHOTOS_DESCRIPTION_LENGHT) {
      throw new PhotoException(ServerErrorType.PARAM_DESCRIPTION_LENGTH);
    }

    this.description = description;
  }

  public void setTitle(String title) throws PhotoException {
    if (title.length() > PicheeseConst.PHOTOS_TITLE_LENGHT) {
      throw new PhotoException(ServerErrorType.PARAM_TITLE_LENGTH);
    }

    this.title = title;
  }

  public void setImageContent(ImageContent imageContent) {
    this.mimeType = imageContent.getMimeType();
    this.imageContent = imageContent;
  }

  public void increaseScore() {
    this.score++;
  }

  public void decreaseScore() {
    this.score--;
  }

  @Override
  public Photo getPojos() {
    String ext = MimeType.getImageExtension(getMimeType());

    PhotoUrl url = new PhotoUrl(
      ImageStorage.getFullPhotoUrl(getId(), ext, "thumbnail"),
      ImageStorage.getFullPhotoUrl(getId(), ext, "normal"),
      ImageStorage.getFullPhotoUrl(getId(), ext, "original")
    );

    User user = null;

    if (getUserLogic() != null) {
      user = getUserLogic().getPojos();
    }

    return new Photo(getId(), url, getRating(), user, getTitle(), getDescription());
  }

  @Override
  public void onCreate() {
    try {
      if (imageContent != null) {
        ImageStorage.getInstance().saveImage(getId(), imageContent);
      }
    } catch (IOException e) {
      //TODO log
    }
  }

  public String getMimeType() {
    return mimeType;
  }

  public ImageContent getImageContent() {
    return imageContent;
  }
}
