package net.stepniak.api.picheese.services;

import net.stepniak.api.entities.BaseEntity;
import net.stepniak.api.error.exception.ServerResponseException;
import net.stepniak.api.picheese.logic.UserLogic;
import net.stepniak.api.picheese.validator.KeyValidator;
import net.stepniak.api.picheese.validator.SessionValidator;
import net.stepniak.api.services.BaseService;
import net.stepniak.picheese.error.server.ServerErrorType;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Produces(MediaType.APPLICATION_JSON + "; charset=utf-8")
//@Produces(MediaType.APPLICATION_JSON)
public abstract class PicheeseBaseService<T extends BaseEntity> extends BaseService<T> {

  private static final Pattern sessionIdHeaderPattern = Pattern.compile("^OAuth\\d? (.+)$");

  @Autowired
  private KeyValidator keyValidator;

  @Autowired
  private SessionValidator sessionValidator;

  protected String validateSessionId(String sessionId) {
    if (StringUtils.isEmpty(sessionId)) {
      sessionId = getSessionId();
    }

    if (StringUtils.isEmpty(sessionId)) {
      throw new ServerResponseException(uriInfo, ServerErrorType.PARAM_SESSION_ID);
    }

    if (!sessionValidator.isValid(sessionId)) {
      throw new ServerResponseException(uriInfo, ServerErrorType.INVALID_SESSION_ID);
    }

    return sessionId;
  }

  protected UserLogic getAuthorizedUser(String sessionId) {
    sessionId = validateSessionId(sessionId);
    UserLogic userLogic = sessionValidator.getUserLogic(sessionId);

    if (userLogic == null) {
      throw new ServerResponseException(uriInfo, ServerErrorType.UNAUTHORIZED_SESSION_ID);
    }

    return userLogic;
  }

  private String getSessionId() {
    String authHeader = request.getHeader("Authorization");
    if (authHeader != null) {
      Matcher matcher = sessionIdHeaderPattern.matcher(authHeader);

      if (matcher.matches()) {
        return matcher.group(1);
      }
    }

    List<String> sessionIdList = uriInfo.getQueryParameters().get("sessionId");

    if (sessionIdList != null && !sessionIdList.isEmpty()) {
      return sessionIdList.get(0);
    }

    return null;
  }

  protected void validateApiKey(String key) {
    if (StringUtils.isEmpty(key)) {
      key = getApiKey();
    }

    if (StringUtils.isEmpty(key)) {
      throw new ServerResponseException(uriInfo, ServerErrorType.PARAM_KEY);
    }

    if (!keyValidator.isValid(key)) {
      throw new ServerResponseException(uriInfo, ServerErrorType.INVALID_KEY);
    }
  }

  private String getApiKey() {
    List<String> keyList = uriInfo.getQueryParameters().get("key");

    if (keyList != null && !keyList.isEmpty()) {
      return keyList.get(0);
    }

    return null;
  }
}
